#include "Vending_machine.h"
#include <catch.h>
#include <unordered_set>

TEST_CASE("Initial balance is 0")
{
    Vending_machine machine;
    CHECK(0 == machine.get_balance());
}

TEST_CASE("After inserting 25, balance is 25")
{
    Vending_machine machine;
    machine.insert_coin(25);
    CHECK(25 == machine.get_balance());
}

TEST_CASE("get_inventory is as expected")
{
    Vending_machine machine;
    CHECK(8 == machine.get_inventory("Snickers"));
    CHECK(4 == machine.get_inventory("Skittles"));
}

TEST_CASE("get_inventory of unknown item throws")
{
    Vending_machine machine;
    CHECK_THROWS_AS(machine.get_inventory("Cheetos"), Unknown_item);
}

TEST_CASE("get_price is as expected")
{
    Vending_machine machine;
    CHECK(125 == machine.get_price("Snickers"));
    CHECK(100 == machine.get_price("Skittles"));
}

TEST_CASE("get_price of unknown item throws")
{
    Vending_machine machine;
    CHECK_THROWS_AS(machine.get_price("Cheetos"), Unknown_item);
}

TEST_CASE("get_items() returns expected set of items")
{
    Vending_machine machine;
    std::vector<std::string> items = machine.get_items();

    std::unordered_set<std::string> actual(items.begin(), items.end());
    std::unordered_set<std::string> expected{"Snickers", "Skittles"};
    CHECK(expected == actual);
}

TEST_CASE("insert_coin adds correct ammount")
{
    Vending_machine machine;

    machine.insert_coin(25);
    CHECK(25 == machine.get_balance());

    machine.insert_coin(10);
    CHECK(35 == machine.get_balance());
}

TEST_CASE("can purchase Snickers")
{
    Vending_machine machine;

    machine.insert_coin(200);
    CHECK(75 == machine.select_item("Snickers"));
    CHECK(0 == machine.get_balance());
}

TEST_CASE("cannot purchase Cheetos because unknown item")
{
    Vending_machine machine;
    machine.insert_coin(1000);
    CHECK_THROWS_AS(machine.select_item("Cheetos"), Unknown_item);
}

TEST_CASE("Cannot purchase Skittles because insufficient funds")
{
    Vending_machine machine;
    machine.insert_coin(1);
    CHECK_THROWS_AS(machine.select_item("Skittles"), Insufficient_funds);
}

TEST_CASE("Repeat select_item() until out of stock exception")
{
    Vending_machine machine;

    auto item  = machine.get_items().front();
    auto price = machine.get_price(item);
    auto count = machine.get_inventory(item);

    while (count--) {
        machine.insert_coin(price);
        CHECK(0 == machine.select_item(item));
    }

    machine.insert_coin(price);
    CHECK_THROWS_AS(machine.select_item(item), Out_of_stock);
}

TEST_CASE("return_change() works")
{
    Vending_machine machine;

    machine.insert_coin(10);
    CHECK(10 == machine.get_balance());
    CHECK(10 == machine.return_change());
    CHECK(0 == machine.get_balance());
}

